@@ -138,7 +138,13 @@ void on_deviceRescan_activate(GtkMenuItem *menuitem, gpointer user_data) {
  */
 void on_filesAdd_activate(GtkMenuItem *menuitem, gpointer user_data) {
     GSList* files;
-
+    // Set the Playlist ID to be asked if needed.
+    if(Preferences.auto_add_track_to_playlist == TRUE){
+        addTrackPlaylistID = GMTP_REQUIRE_PLAYLIST;
+    } else {
+        addTrackPlaylistID = GMTP_NO_PLAYLIST;
+    }
+    // Get the files, and add them.
     files = getFileGetList2Add();
     if (files != NULL)
         g_slist_foreach(files, (GFunc) __filesAdd, NULL);
@@ -424,6 +430,46 @@ void on_PrefsAskDownload_activate(GtkMenuItem *menuitem, gpointer user_data) {
 
 // ************************************************************************************************
 
+/**
+ * Callback for Ask Download Path Operations toggle in Preferences Dialog Box.
+ * @param menuitem
+ * @param user_data
+ */
+void on_PrefsAutoAddTrackPlaylist_activate(GtkMenuItem *menuitem, gpointer user_data) {
+    gboolean state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbuttonAutoAddTrackPlaylist));
+
+#if GMTP_USE_GTK2
+    if (gconfconnect != NULL)
+        gconf_client_set_bool(gconfconnect, "/apps/gMTP/autoAddTrackPlaylist", state, NULL);
+#else
+    if (gsettings_connect != NULL)
+        g_settings_set_boolean(gsettings_connect, "autoaddtrackplaylist", state);
+    g_settings_sync();
+#endif
+} // end on_PrefsAutoAddTrackPlaylist_activate()
+
+// ************************************************************************************************
+
+/**
+ * Callback for Ignore Path in Playlist toggle in Preferences Dialog Box.
+ * @param menuitem
+ * @param user_data
+ */
+void on_PrefsIgnorePathInPlaylist_activate(GtkMenuItem *menuitem, gpointer user_data) {
+    gboolean state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbuttonIgnorePathInPlaylist));
+
+#if GMTP_USE_GTK2
+    if (gconfconnect != NULL)
+        gconf_client_set_bool(gconfconnect, "/apps/gMTP/ignorepathinplaylist", state, NULL);
+#else
+    if (gsettings_connect != NULL)
+        g_settings_set_boolean(gsettings_connect, "ignorepathinplaylist", state);
+    g_settings_sync();
+#endif
+} // end on_PrefsIgnorePathInPlaylist_activate()
+
+// ************************************************************************************************
+
 /**
  * Callback for Confirm Overwrite of File Operations toggle in Preferences Dialog Box.
  * @param menuitem
@@ -533,7 +579,7 @@ void fileListRowActivated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewC
     // Obtain the iter, and the related objectID.
     model = gtk_tree_view_get_model(treeview);
     if (gtk_tree_model_get_iter(model, &iter, path)) {
-        gtk_tree_model_get(GTK_TREE_MODEL(fileList), &iter, COL_ISFOLDER, &isFolder, COL_FILENAME, &filename, COL_FILEID, &objectID, -1);
+        gtk_tree_model_get(GTK_TREE_MODEL(fileList), &iter, COL_ISFOLDER, &isFolder, COL_FILENAME_ACTUAL, &filename, COL_FILEID, &objectID, -1);
         if (isFolder == FALSE) {
             // Now download the actual file from the MTP device.
             filesDownload(filename, objectID);
@@ -1013,6 +1059,134 @@ void on_Playlist_NewPlaylistButton_activate(GtkMenuItem *menuitem, gpointer user
 
 // ************************************************************************************************
 
+/**
+ * Callback to handle the Import Playlist button in the Playlist editor dialog.
+ * @param menuitem
+ * @param user_data
+ */
+void on_Playlist_ImportPlaylistButton_activate(GtkMenuItem *menuitem, gpointer user_data) {
+    //g_printf("Clicked on new playlist button\n");
+    gchar *playlistfilename = NULL;
+    gchar *playlistname = NULL;
+    GtkWidget *FileDialog;
+    GtkFileFilter *OpenFormFilter, *OpenFormFilter2;
+
+    // Save our current selected playlist!
+    if (devicePlayLists != NULL)
+        playlist_SavePlaylist(playlist_number);
+
+    // Get our filename...
+
+    FileDialog = gtk_file_chooser_dialog_new(_("Select Playlist to Import"),
+        GTK_WINDOW(windowMain), GTK_FILE_CHOOSER_ACTION_OPEN,
+        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+        NULL);
+    gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(FileDialog), FALSE);
+    OpenFormFilter = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(OpenFormFilter, "*.m3u");
+    gtk_file_filter_set_name(OpenFormFilter, "m3u Playlists");
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileDialog), OpenFormFilter );
+    OpenFormFilter2 = gtk_file_filter_new();
+    gtk_file_filter_add_pattern(OpenFormFilter2, "*");
+    gtk_file_filter_set_name(OpenFormFilter2, "All Files");
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(FileDialog), OpenFormFilter2 );
+
+    if (gtk_dialog_run(GTK_DIALOG(FileDialog)) == GTK_RESPONSE_ACCEPT) {
+        playlistfilename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileDialog));
+    }
+
+    gtk_widget_hide(FileDialog);
+    gtk_widget_destroy(FileDialog);
+
+    if (playlistfilename != NULL) {
+        // Add in playlist to MTP device.
+        playlistname = playlistImport(playlistfilename);
+
+        // If our name is NULL, then the import failed...
+        if(playlistname != NULL){
+            // Refresh our playlist information.
+            devicePlayLists = getPlaylists();
+            gtk_list_store_clear(GTK_LIST_STORE(playlist_PL_List));
+            // Add it to our combobox
+
+    #if GMTP_USE_GTK2
+            gtk_combo_box_append_text(GTK_COMBO_BOX(comboboxentry_playlist), g_strdup(playlistname));
+    #else
+            gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(comboboxentry_playlist), g_strdup(playlistname));
+    #endif
+
+            // Set the active combobox item.
+            comboboxentry_playlist_entries++;
+            playlist_number = comboboxentry_playlist_entries - 1;
+            gtk_combo_box_set_active(GTK_COMBO_BOX(comboboxentry_playlist), comboboxentry_playlist_entries - 1);
+            SetPlaylistButtonState(TRUE);
+            setPlaylistField(playlist_number);
+
+            // Clean up fields.
+            g_free(playlistname);
+        } else {
+            // Let the user know the import failed.
+            g_fprintf(stderr, _("The playlist failed to import correctly.\n"));
+            displayError(_("The playlist failed to import correctly.\n"));
+        }
+        // Clean up fields.
+        g_free(playlistfilename);
+    }
+} // end on_Playlist_ImportPlaylistButton_activate()
+
+// ************************************************************************************************
+
+/**
+ * Callback to handle the Export Playlist button in the Playlist editor dialog.
+ * @param menuitem
+ * @param user_data
+ */
+void on_Playlist_ExportPlaylistButton_activate(GtkMenuItem *menuitem, gpointer user_data){
+    gchar *playlistfilename = NULL;
+    GtkWidget *FileDialog;
+
+    // Save our current selected playlist!
+    if (devicePlayLists != NULL)
+        playlist_SavePlaylist(playlist_number);
+
+    gint PlayListID = gtk_combo_box_get_active(GTK_COMBO_BOX(comboboxentry_playlist));
+
+    if (PlayListID != -1) {
+        // We have something selected so lets do the dance.
+        LIBMTP_playlist_t* tmpplaylist = devicePlayLists;
+        if (PlayListID > 0) {
+            while (PlayListID--)
+                if (tmpplaylist->next != NULL)
+                    tmpplaylist = tmpplaylist->next;
+        }
+        // We should be in the correct playlist LIBMTP structure.
+
+        playlistfilename = g_strdup_printf("%s.%s", tmpplaylist->name, "m3u");
+
+        FileDialog = gtk_file_chooser_dialog_new(_("Save as..."),
+        GTK_WINDOW(windowMain), GTK_FILE_CHOOSER_ACTION_SAVE,
+        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+        GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+        NULL);
+        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(FileDialog), playlistfilename );
+
+        if (gtk_dialog_run(GTK_DIALOG(FileDialog)) == GTK_RESPONSE_ACCEPT) {
+            playlistfilename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(FileDialog));
+        }
+
+        gtk_widget_hide(FileDialog);
+        gtk_widget_destroy(FileDialog);
+
+        if (playlistfilename != NULL) {
+            playlistExport(playlistfilename, tmpplaylist);
+            g_free(playlistfilename);
+        }
+    }
+} // end on_Playlist_ExportPlaylistButton_activate()
+
+// ************************************************************************************************
+
 /**
  * Callback to handle the Delete Playlist button in the Playlist editor dialog.
  * @param menuitem
@@ -1176,3 +1350,37 @@ void on_progressDialog_Cancel(GtkWidget *button, gpointer user_data) {
     gtk_widget_hide(progressDialog);
     gtk_widget_destroy(progressDialog);
 } // end on_progressDialog_Cancel()
+
+
+// ************************************************************************************************
+
+/**
+ * Callback to handle user asking to create a new playlist from the AutoAddTrack to Playlist option.
+ * @param button
+ * @param user_data
+ */
+void on_TrackPlaylist_NewPlaylistButton_activate(GtkWidget *button, gpointer user_data) {
+    gchar *playlistname = NULL;
+    gint combobox_entries = 0;
+
+    playlistname = displayPlaylistNewDialog();
+    if (playlistname != NULL) {
+        // Add in playlist to MTP device.
+        playlistAdd(playlistname);
+        // Refresh our playlist information.
+        devicePlayLists = getPlaylists();
+
+        // Add it to our combobox
+#if GMTP_USE_GTK2
+        gtk_combo_box_append_text(GTK_COMBO_BOX(combobox_AddTrackPlaylist), g_strdup(playlistname));
+#else
+        gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combobox_AddTrackPlaylist), g_strdup(playlistname));
+#endif
+        g_free(playlistname);
+
+        // Set the active combobox item.
+        combobox_entries = gtk_tree_model_iter_n_children(gtk_combo_box_get_model(GTK_COMBO_BOX(combobox_AddTrackPlaylist)), NULL);
+        gtk_combo_box_set_active(GTK_COMBO_BOX(combobox_AddTrackPlaylist), combobox_entries - 1);
+    }
+}
+
