@@ -113,6 +113,8 @@ extern "C" {
 
     gboolean folderListRemove(GList *List);
 
+    // Flags for overwriting files of host PC and device.
+    gint fileoverwriteop;
     // Flag to allow overwrite of files on device.
     gint deviceoverwriteop;
     // Aggreegate function for adding a file to the device.
@@ -180,6 +182,9 @@ extern "C" {
 
     void playlist_SavePlaylist(gint PlayListID);
 
+    // Add track to playlist dialog;
+    uint32_t displayAddTrackPlaylistDialog(void);
+
     // Widgets for menu items;
     GtkWidget *fileConnect;
     GtkWidget *fileAdd;
@@ -226,6 +231,8 @@ extern "C" {
     GtkWidget *checkbuttonDownloadPath;
     GtkWidget *checkbuttonConfirmFileOp;
     GtkWidget *checkbuttonConfirmOverWriteFileOp;
+    GtkWidget *checkbuttonAutoAddTrackPlaylist;
+    GtkWidget *checkbuttonIgnorePathInPlaylist;
 
     // AlbumArt Dialog global pointers
     GtkWidget *AlbumArtDialog;
@@ -245,6 +252,7 @@ extern "C" {
 
     // Buttons for playlist
     GtkWidget *button_Del_Playlist;
+    GtkWidget *button_Export_Playlist;
     GtkWidget *button_File_Move_Up;
     GtkWidget *button_File_Move_Down;
     GtkWidget *button_Del_File;
@@ -253,6 +261,9 @@ extern "C" {
     // Widget for formatDevice progress bar.
     GtkWidget *formatDialog_progressBar1;
 
+    // Combobox used in AddTrackPlaylist feature.
+    GtkWidget *combobox_AddTrackPlaylist;
+
 #ifdef  __cplusplus
 }
 #endif
